#include <so_5/all.hpp>

// Сигналы, которыми будут обмениваться pinger и ponger.
struct ping final : public so_5::signal_t {};
struct pong final : public so_5::signal_t {};

// Агент pinger, первым отсылает ping, затем отсылает ping в ответ на pong.
class pinger final : public so_5::agent_t {
   // Почтовый ящик, через который будет происходить обмен сообщениями.
   const so_5::mbox_t mbox_;
public:
   // Конструктор получает почтовый ящик в качестве аргумента.
   pinger(context_t ctx, so_5::mbox_t mbox)
         :  so_5::agent_t{std::move(ctx)}
         ,  mbox_{std::move(mbox)} {
      // Сразу подписываем агента на нужный сигнал.
      so_subscribe(mbox_).event([this](mhood_t<pong>) {
         std::cout << "pong!" << std::endl;
         so_5::send<ping>(mbox_);
      });
   }

   // В самом начале своей работы отсылаем первый сигнал ping.
   virtual void so_evt_start() override {
      so_5::send<ping>(mbox_);
   }
};

// Агент ponger, только отвечает pong-ами в ответ на ping-и.
class ponger final : public so_5::agent_t {
   // Почтовый ящик, через который будет происходить обмен сообщениями.
   const so_5::mbox_t mbox_;
public:
   // Конструктор получает почтовый ящик в качестве аргумента.
   ponger(context_t ctx, so_5::mbox_t mbox)
         :  so_5::agent_t{std::move(ctx)}
         ,  mbox_{std::move(mbox)} {
      // Сразу подписываем агента на нужный сигнал.
      so_subscribe_self().event([this](mhood_t<ping>) {
         std::cout << "ping!" << std::endl;
         so_5::send<pong>(mbox_);
      });
   }
};

int main() {
   // Запускаем SObjectizer с двумя агентами в рамках одной кооперации.
   so_5::launch([](so_5::environment_t & env) {
         env.introduce_coop([](so_5::coop_t & coop) {
            // Почтовый ящик, который потребуется агентам.
            const auto mbox = coop.environment().create_mbox();
            // Сами агенты.
            coop.make_agent<pinger>(mbox);
            coop.make_agent<ponger>(mbox);
         });
      },
      [](so_5::environment_params_t & params) {
         // Разрешаем трассировку механизма доставки сообщений.
         // Направляем трассировочные сообщения на стандартый поток вывода.
         params.message_delivery_tracer(so_5::msg_tracing::std_cout_tracer());
      });
}

